#!/usr/bin/env python
# -*- coding: iso-8859-15 -*-

#!/usr/bin/env python

# GIMP plugin to swap pixels across a path
# (c) Ofnuts 2011
#
#   History:
#
#   v0.0: first published version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


import math, random
from gimpfu import *

traceFlag=True

def trace(msg):
    if traceFlag:
        pdb.gimp_message(msg)


def computePixelsFor(x,y,slope,distance,quadratic):

	deltaX=deltaY=0
	if math.fabs(slope) < 1/1000.:
		deltaX=0
		deltaY=1
	elif math.fabs(slope) > 1000.:
		deltaX=1
		deltaY=0
	else:
		perpSlope=-1/slope
		deltaX=1/math.sqrt(1+(perpSlope*perpSlope))
		deltaY=perpSlope*deltaX
	
	r1=random.random()
	r2=random.random()
	if quadratic:
		d1=distance*r1*r1
		d2=distance*r2*r2
	else:
		d1=distance*r1
		d2=distance*r2
	
	x1=int(round(x+deltaX*d1))
	y1=int(round(y+deltaY*d1))
	x2=int(round(x-deltaX*d2))
	y2=int(round(y-deltaY*d2))
	
	return (x1,y1,x2,y2)


def getPixel(drawable,x,y):
	"""Gets the values of a given pixel"""
	(bpp,(channels))=pdb.gimp_drawable_get_pixel(drawable,x,y)
	if (bpp==4):
		(r,g,b,a)=channels
	elif (bpp==3):
		(r,g,b)=channels
		a=255
	elif (bpp==2):
		(v,a)=channels
		r=g=b=v
	elif (bpp==1):
		v=channels
		r=g=b=v
		a=255
	return [r,g,b,a]

def isInsideDrawable(x,y,drawable):
	return isInsideCoords(x,y,0,0,drawable.width-1,drawable.height-1)

def isInsideCoords(x,y,minX,minY,maxX,maxY):
	return x >= minX and y >= minY and x <= maxX  and y <= maxY

def setPixel(drawable,x,y,bpp,values):
	if isInsideDrawable(x,y,drawable):
		pdb.gimp_drawable_set_pixel(drawable, x,y,bpp,values)

def path_test(image,drawable,width,density,spread):
	activeLayer=pdb.gimp_image_get_active_layer(image)
	path = pdb.gimp_image_get_active_vectors(image)
	if path==None:
		pdb.gimp_message('No active path in image')
		return
	#print "---------------------------------------"
	#print spread
	#print path.strokes
	
	quadratic=spread=='Quadratic'
	
	totalLength=0
	for stroke in path.strokes:
		length=pdb.gimp_vectors_stroke_get_length(path, stroke.ID, .1)
		totalLength=totalLength+length
	
	#print totalLength
	
	dots=gimp.Layer(image,'Fuzzdots',drawable.width,drawable.height,RGBA_IMAGE,100, NORMAL_MODE)
	image.add_layer(dots,0)
	
	# more or less the # of runs per one-pixel length of path
	repeat=1+(width*density/200)
	
	doneSoFar=0
	step=1/repeat
	
	for stroke in path.strokes:
		length=pdb.gimp_vectors_stroke_get_length(path, stroke.ID, .1)
		
		atDist=0
		while atDist < length: 
			gimp.progress_update(1.*doneSoFar/totalLength)
	
			x,y,slope,valid=pdb.gimp_vectors_stroke_get_point_at_dist(path, stroke.ID,atDist, .1)
			x1,y1,x2,y2=computePixelsFor(x,y,slope,width,quadratic)
			if isInsideDrawable(x1,y1,drawable) and isInsideDrawable(x2,y2,drawable):
				setPixel(dots,x1,y1,4,getPixel(drawable,x2,y2))
				setPixel(dots,x2,y2,4,getPixel(drawable,x1,y1))
			atDist=atDist+step
			doneSoFar=doneSoFar+step

	pdb.gimp_displays_flush()
	pdb.gimp_drawable_update(dots, 0,0,dots.width,dots.height)
	pdb.gimp_image_set_active_layer(image,activeLayer)



### Registration
    
register(
	"fuzzpixels",
	N_("Swap pixels across path..."),
	"Swap pixels across path...",
	"Ofnuts",
	"Ofnuts",
	"2011",
	N_("Swap pixels across path..."),
	"RGB*",
	[
		(PF_IMAGE, "image", "Input image", None),
		(PF_DRAWABLE, "drawable", "Input drawable", None),
		(PF_SPINNER, "width", "Width", 50, (1, 500, 1)),
		(PF_SLIDER, "density",  "Density", 0, (0, 100, 1)),
		(PF_RADIO, "spread", ("Spread"), "Linear", (("Linear","Linear"),("Quadratic","Quadratic")))
	],
	[],
	path_test,
	menu="<Image>/Filters/Noise",
	domain=("gimp20-python", gimp.locale_directory)
)


main()
